use std::sync::{Arc, Mutex};
use embedded_graphics::{
    prelude::*,
    mono_font::{ascii::FONT_7X13, MonoTextStyle},
    pixelcolor::BinaryColor,
    text::{Text, Alignment},
};
use esp_idf_hal::i2c::I2cDriver;
use ssd1306::{prelude::*, Ssd1306};
use crate::error::Error;

pub fn update_display<T>(interface: I2CInterface<I2cDriver<'_>>, data: Arc<Mutex<T>>) -> Result<(), Error>
where T: ToString
{
    let mut display = Ssd1306::new(interface, DisplaySize128x64, DisplayRotation::Rotate0)
        .into_buffered_graphics_mode();

    display.init()
        .map_err(|_| Error::DisplayError(DisplayError::SetupError))?;

    // let raw: ImageRaw<BinaryColor> = ImageRaw::new(include_bytes!("./../rust.raw"), 64);
    // let im: Image<'_, _> = Image::new(&raw, Point::new(32, 0));
    // im.draw(&mut display).unwrap();

    let style = MonoTextStyle::new(&FONT_7X13, BinaryColor::On);
    
    display.clear(BinaryColor::Off)
        .map_err(|_| Error::DisplayError(DisplayError::DrawingError))?;
    
    Text::with_alignment(
        &data.lock().unwrap().to_string().as_str(),
        Point::new(64, 10),
        style,
        Alignment::Center,
    )
        .draw(&mut display)
        .map_err(|_| Error::DisplayError(DisplayError::DrawingError))?;

    display.flush().map_err(|_| Error::DisplayError(DisplayError::FlushError))?;
    Ok(())
}

/// Enum to make clear the stage of failure of the display
pub enum DisplayError {
    SetupError,
    DrawingError,
    FlushError,
}