use esp_idf_sys::{self as _, EspError}; // If using the `binstart` feature of `esp-idf-sys`, always keep this module imported
use esp_idf_hal::{
    prelude::Peripherals, 
    gpio::{AnyInputPin, AnyOutputPin},
    delay::BLOCK,
    units::Hertz,
    uart,
    i2c::{I2cConfig, I2cDriver} 
};
use esp_idf_hal;
use ssd1306::I2CDisplayInterface;

mod error;
mod gps;
use gps::gpsdata::GpsData;

mod display;




fn main() -> Result<(), EspError> {
    // It is necessary to call this function once. Otherwise some patches to the runtime
    // implemented by esp-idf-sys might not link properly. See https://github.com/esp-rs/esp-idf-template/issues/71
    esp_idf_sys::link_patches();
    // Bind the log crate to the ESP Logging facilities
    esp_idf_svc::log::EspLogger::initialize_default();

    // Get Peripherals and sysloop ready
    let peripherals = Peripherals::take().unwrap();
    let pins = peripherals.pins;

    // Define program's pins + addresses
    let scl = pins.gpio22;
    let sda = pins.gpio21;
    let gps_rx  = pins.gpio16;
    let _lcd_address: u8 = 0x3C;

    
    // Setup UART to read serial data from GPS
    let config = uart::config::Config::default().baudrate(Hertz(9_600));
    let uart: uart::UartRxDriver = uart::UartRxDriver::new(
        peripherals.uart1,
        gps_rx,
        None::<AnyInputPin>,
        None::<AnyOutputPin>,
        &config
    )?;

    // Setup I2C driver
    let config = I2cConfig::new().baudrate(Hertz(921600));
    let i2c = I2cDriver::new(
        peripherals.i2c0, 
        sda, 
        scl, 
        &config
    )?;
    let interface = I2CDisplayInterface::new(i2c);

    let mut latest_data = GpsData::new();
    
    loop {
        // Read buffer from UART
        let mut buf: Vec<u8> = (0..uart.count()? as u8).collect();
        uart.read(&mut buf[..], BLOCK)?;
        std::thread::sleep(std::time::Duration::from_millis(10));

        
    }
}


